#!/usr/bin/env python3
import os
import time

from cnoid.Util import *
from cnoid.Base import *
from cnoid.Body import *
from cnoid.BodyPlugin import *
# from cnoid.ROSPlugin import *
import math
import yaml

item_list = []
robot_item = []
env_item = []
env_folder = []
sim_item = []
world_item = WorldItem()

def readYaml(filepath):
    try:
        global item_list
        # script_dir = os.path.dirname(__file__)
        # param_dir = os.path.join(script_dir, '../param')
        # filepath = param_dir + '/cnoid_item_list.yaml'
        # Read yaml file.
        with open(filepath, 'r') as f:
            item_list = yaml.safe_load(f)
    except Exception as e:
        print('[ERROR] Specified file is not found.')

def sceneViewProperties():
    # Set view properties.
    sceneWidget = SceneView.instance.sceneWidget
    sceneWidget.setHeadLightEnabled(True)
    sceneWidget.setHeadLightIntensity(0.1)
    sceneWidget.setFloorGridEnabled(False)
    sceneWidget.setWorldLightIntensity(0.25)
    sceneWidget.setWorldLightAmbient(0.25)
    sceneWidget.setBackgroundColor([0.1, 0.1, 0.3])
    # sceneWidget.setCameraPosition(
    #     [ 0.5, 10.5, 1.4 ],
    #     [ -0.07, -0.98, -0.15 ],
    #     [ -0.01, -0.15, 0.98 ])
    sceneWidget.setCameraPosition(
        [-12.5, -7.0, 12.0],
        [1.57, 0.0, -1.04],
        [-0.01, -0.015, 1.7])


def worldItemGenerator():
    global world_item
    # Generate world item.
    RootItem.instance.addChildItem(world_item)

def readRobotModel():
    global item_list, robot_item
    
    if checkKey("ROS", item_list):
        # Read WorldROS item.
        readWorldRos()
    try:
        readBodyModel(item_list, robot_item, 'Robot')
    except Exception as e:
        print("[Error]", e)

def readBodyModel(item, body_item, e1 = ""):
    folder = []
    try:
        global world_item
        # Number of body models.
        body_num = len(item[e1])
        folder = []
        for i in range(body_num):
            if not checkKey("uri", item, e1, i):
                folder_item = FolderItem()
                folder_item.name = item[e1][i]['name']
                parent = item[e1][i]['parent']
                body_ret = checkParentKey(parent, body_item)
                folder_ret = checkParentKey(parent, folder)
                if parent == "World":
                    # folder is NULL
                    world_item.addChildItem(folder_item)
                    folder.append(folder_item)
                elif len(body_item) > 0 and body_ret[0]:
                    num = body_ret[1]
                    body_item[num].addChildItem(folder_item)
                    folder.append(folder_item)
                elif len(folder) > 0 and folder_ret[0]:
                    # Otherwise
                    num = folder_ret[1]
                    folder[num].addChildItem(folder_item)
                    folder.append(folder_item)
#                 body_item.append(folder_item)
            else:
                # Generate BodyItem.
                tmp = BodyItem()
                # Load body model.
                tmp.load(item[e1][i]['uri'])
                tmp.name = item[e1][i]['name']
                # Display robot model.
                tmp.setChecked(True)
                body = tmp.body
                if checkKey("translation", item, e1, i):
                    # Check existence of "translation" key.
                    body.rootLink.setTranslation(item[e1][i]['translation'])
                else:
                    # Otherwise
                    body.rootLink.setTranslation([ 0.0, 0.0, 0.0 ])
                    
                if checkKey("rotation", item, e1, i):
                    # Check existence of "rotation" key.
                    r = math.radians(item[e1][i]['rotation'][0])
                    p = math.radians(item[e1][i]['rotation'][1])
                    y = math.radians(item[e1][i]['rotation'][2])
                else:
                    # Otherwise
                    r = p = y = 0.0
                
                body.rootLink.setRotation(rotFromRpy([r, p, y]))
                body.calcForwardKinematics()
                tmp.storeInitialState()
                
                if not checkKey("parent", item, e1, i) or item_list[e1][i]['parent'] == "World":
                    # Check existence of "parent" key. Or "parent" key is "World"
                    world_item.addChildItem(tmp)
                else:
                    # Otherwise
                    # Parent item name.
                    parent = item_list[e1][i]['parent']
                    # Parent key existence check.
                    body_ret = checkParentKey(parent, body_item)
                    folder_ret = checkParentKey(parent, folder)
                    if len(body_item) > 0 and body_ret[0]:
                        # if it exists in the body item list.
                        num = body_ret[1]
                        body_item[num].addChildItem(tmp)
                    elif len(folder) > 0 and folder_ret[0]:
                        # if it exists in the folder item list.
                        num = folder_ret[1]
                        folder[num].addChildItem(tmp)
                    else:
                        # Otherwise
                        folder_item = FolderItem()
                        folder_item.name = item_list[e1][i]['parent']
                        world_item.addChildItem(folder_item)
                        folder_item.addChildItem(tmp)
                        folder.append(folder_item)
                # Add body item list.
                body_item.append(tmp)
        
                if checkKey("controller", item, e1, i):
                    # Check existence of "controller" key.
                    controller_item = SimpleControllerItem()
                    controller_item.name = item[e1][i]['controller']
                    controller_item.setController(controller_item.name)
                    if checkKey("controllerOption", item, e1, i):
                        # Check existence of "controllerOption" key.
                        controller_item.setOptions(item[e1][i]['controllerOption'])
                    body_item[i].addChildItem(controller_item)
    
                if e1 == "Robot" and checkKey("ROS", item):
                    # Read BodyROS item.
                    readBodyRos(body_item, i)
    except Exception as e:
        print("[ERROR]", e)
    return folder


def checkKey(key, item, e1 = "", e2 = -1):
    try:
        if not e1:
            if not key in item:
                return False
        elif e1 and e2 == -1:
            if not key in item[e1]:
                return False
        else:
            if not key in item[e1][e2]:
                return False
        return True
    except Exception as e:
        print("[ERROR]", e)

def checkParentKey(parent, body_item):
    try:
        length = len(body_item)
        for i in range(length):
            if parent in body_item[i].name:
                return True, i
        return False, -1
    except Exception as e:
        print("[ERROR]", e)

def readWorldRos():
    global item_list, world_item
    
    try:
        worldRosItem = SubProjectItem()
        key = 'WorldROS'
        idx = 0
        if checkKey('uri', item_list, 'ROS', 0) and key in item_list['ROS'][0]['uri']:
            idx = 0
        elif checkKey('uri', item_list, 'ROS', 1) and key in item_list['ROS'][1]['uri']:
            idx = 1
            
        if not checkKey('name', item_list, 'ROS', idx):
            worldRosItem.name = "WorldROS"
        else:
            worldRosItem.name = item_list['ROS'][idx]['name']
        if not checkKey('uri', item_list, 'ROS', idx):
            print("[ERROR] Specified file is not found.")
        else:
            worldRosItem.load(item_list['ROS'][idx]['uri'])
            world_item.addChildItem(worldRosItem)
    except Exception as e:
        print("[Error]", e)

def readBodyRos(body_item, num):
    global item_list
    
    try:
        bodyRosItem = SubProjectItem()
        key = 'BodyROS'
        idx = 0
        if checkKey('uri', item_list, 'ROS', 0) and key in item_list['ROS'][0]['uri']:
            idx = 0
        elif checkKey('uri', item_list, 'ROS', 1) and key in item_list['ROS'][1]['uri']:
            idx = 1
            
        if not checkKey('name', item_list, 'ROS', idx):
            bodyRosItem.name = "BodyROS"
        else:
            bodyRosItem.name = item_list['ROS'][idx]['name']
        if not checkKey('uri', item_list, 'ROS', idx):
            print("[ERROR] Specified file is not found.")
        else:
            bodyRosItem.load(item_list['ROS'][idx]['uri'])
            body_item[num].addChildItem(bodyRosItem)
    except Exception as e:
        print("[Error]", e)

def readEnvironmentModel():
    global item_list, env_item, env_folder

    try:
        env_folder = readBodyModel(item_list, env_item, 'Environment')
    except Exception as e:
        print("[Error]", e)
    return env_folder


def simulatorItemGenerator(item, e1 = ""):
    global sim_item, item_list
    try:
        if checkKey('type', item, e1, 0):
            if item[e1][0]['type'] == "AISTSimulator":
                simulator_item = AISTSimulatorItem()
                simulator_item.setTimeStep(0.001)
                simulator_item.setRealtimeSyncMode(True)
                simulator_item.setTimeRangeMode(SimulatorItem.TR_UNLIMITED)
                simulator_item.setMaxNumIterations(1000)
                simulator_item.setContactCorrectionVelocityRatio(30)
            elif item[e1][0]['type'] == "ODESimulator":
                simulator_item = SubProjectItem()
                simulator_item.name = item[e1][0]['name']
                if checkKey('uri', item, e1, 0):
                    simulator_item.load(item[e1][0]['uri'])
                print("ODESimulator")
            elif item[e1][0]['type'] == "AGXSimulator":
                simulator_item = SubProjectItem()
                simulator_item.name = item[e1][0]['name']
                if checkKey('uri', item, e1, 0):
                    simulator_item.load(item[e1][0]['uri'])
                print("AGXSimulator")
            else:
                print("[ERROR] Undefined simulator item.")
                return False
            world_item.addChildItem(simulator_item)
            simulator_item.setSelected(True)
            sim_item.append(simulator_item)
            # Generate sub simulator item
            subSimulatorItemGenerator(item_list, 'SubSimulator')
            return True
        else:
            return False
    except Exception as e:
        print("[ERROR]", e)
    
def subSimulatorItemGenerator(item, e1 = ""):
    global world_item
    try:
        subsim_num = len(item[e1])
        for i in range(subsim_num):
            if checkKey('type', item, e1, i) and item[e1][i]['type'] == "GLVisionSimulator":
                subSimulator_item = GLVisionSimulatorItem()
                subSimulator_item.name = item[e1][i]['name']
                if checkKey("target_body", item, e1, i) and item[e1][i]['target_body']:
                    subSimulator_item.setTargetBodies(item[e1][i]['target_body'])
                if checkKey("target_sensor", item, e1, i) and item[e1][i]['target_sensor']:
                    subSimulator_item.setTargetSensors(item[e1][i]['target_sensor'])
                if not sim_item[0].childItem:
                    simulator = sim_item[0]
                else:
                    simulator = sim_item[0].childItem
                simulator.addChildItem(subSimulator_item)
                    
    except Exception as e:
        print("[ERROR]", e)


def set_body_model(item, body_item, view_a=None, folder=None, e1=""):
    """
    Rearranging Body Items
    :param item: Item list
    :param body_item: Existing Body Item
    :param view_a: Item name to be placed
    :param folder: Folder Item list
    :param e1: element name
    :return: Updated folder item list
    """
    try:
        global world_item

        if folder is None:
            folder = []
        if view_a is None:
            view_a = []
        # Number of body models.
        for body_yaml in item[e1]:
            for view_obj in view_a:
                if view_obj.lower() in body_yaml["name"].lower():
                    break
            else:
                continue

            if body_yaml.get("uri") is None:
                continue
            # Generate BodyItem.
            tmp = BodyItem()
            # Load body model.
            tmp.load(body_yaml["uri"])
            tmp.setName(body_yaml["name"])
            print(tmp.name)
            # Display robot model.
            tmp.setChecked(True)
            body = tmp.body

            translation = [0.0, 0.0, 0.0]
            if body_yaml.get("translation") is not None:
                translation = body_yaml["translation"]
            body.rootLink.setTranslation(translation)

            r = p = y = 0.0
            if body_yaml.get("rotation") is not None:
                r = math.radians(body_yaml["rotation"][0])
                p = math.radians(body_yaml["rotation"][1])
                y = math.radians(body_yaml["rotation"][2])

            body.rootLink.setRotation(rotFromRpy([r, p, y]))
            body.calcForwardKinematics()
            tmp.storeInitialState()

            for folder_item in folder:
                if folder_item.name.lower() == "FreeSpace".lower():
                    folder_item.addChildItem(tmp)
                    break
            # Add body item list.
            body_item.append(tmp)
    except Exception as e:
        print("[ERROR]", e)
    return folder


def delete_environment_model(delete_name_a):
    global env_item
    for del_item in env_item:
        detect = [view_object for view_object in delete_name_a if view_object.lower() in del_item.name.lower()]
        if len(detect) != 0:
            del_item.removeFromParentItem()


if __name__ == "__main__":
    try:
        # Read yaml file.
        readYaml()
        # Generate world item.
        worldItemGenerator()
        # Generate simulator item
        simulatorItemGenerator(item_list, 'Simulator')
        # Read robot models.
        readRobotModel()
        # Read environment models.
        readEnvironmentModel()
#         # Generate simulator item
#         simulatorItemGenerator(item_list, 'Simulator')
        # Set scene view properties.
        sceneViewProperties()
        # Start simulation.
        #sim_item[0].startSimulation()
    except Exception as e:
        print("[ERROR]", e)