#!/usr/bin/env python
# coding: utf-8

"""
 *
 * convertQuaternionToRadian.py
 *
 *  Created on: 2022/10/28
 *      Author: T.Anazawa
 *
 *  Outline: Convert quaternion to euler angles.
 *
"""
import os
import sys
import rospy
import csv
import math
import tf
from geometry_msgs.msg import Vector3
from docutils.parsers.rst.directives import encoding
from docutils.utils.punctuation_chars import delimiters

def quaternion_to_euler(quaternion):
    euler = tf.transformations.euler_from_quaternion(quaternion)
    roll = euler[0]
    pitch = euler[1]
    yaw = euler[2]
    
    degree = radian_to_degree(euler)
    
    writeCsv(degree)
    
    print(degree)

def radian_to_degree(euler):
    roll = euler[0] / math.pi * 180.0
    pitch = euler[1] / math.pi * 180.0
    yaw = euler[2] / math.pi * 180.0
    
    degree = [ roll, pitch, yaw ]
    
    return degree

def readCsv(filename):
    try:
        with open(filename, "rb") as csv_file:
            # リスト形式
            f = csv.reader(csv_file, delimiter=",", doublequote=True, lineterminator="\r\n", quotechar='"', skipinitialspace=True)
            header = next(f)
            for row in f:
                quaternion = (float(row[11]), float(row[12]), float(row[13]), float(row[14]))
                quaternion_to_euler(quaternion)
                
    except Exception as err:
        print(err)

def writeCsv(euler):
    try:
        filePath = "./euler.csv"
        if not os.path.isfile(filePath):
            hdr = [ "roll", "pitch", "yaw" ]
            with open(filePath, "a") as f:
                writer = csv.writer(f)
                writer.writerow(hdr)
                
        with open(filePath, "a") as f:
            writer = csv.writer(f)
            writer.writerow([ euler[0], euler[1], euler[2] ])
    
    except Exception as err:
        print(err)

if __name__ == '__main__':
    try:
        args = sys.argv
        filename = args[1]
        print(filename)
        readCsv(filename)
        
    except rospy.ROSInterruptException:
        pass
    
    finally:
        print("finish")