#!/usr/bin/env python
# coding: utf-8

"""
 *
 * slip_test.py
 *
 *  Created on: 2022/09/22
 *      Author: T.Anazawa
 *
 *  Outline: 
 *
"""
import rospy
import time
import math
from geometry_msgs.msg import Twist
from math import pi
import rospy_tutorials
from __builtin__ import False

class slip_test():
    def __init__(self):
        self.pub_ = rospy.Publisher('cmd_vel', Twist, queue_size=1)
    
    def pub_trans_x(self):
        rospy.loginfo('********** pub_trans_x **********')
        # distance
        dist = rospy.get_param('cmdvel_publisher/dist', 0) # [m]
        # velocity
        vel = rospy.get_param('cmdvel_publisher/vel', 0) # [m/s]
        
        rospy.loginfo('dist: ' + str(dist) + ' vel: ' + str(vel))
        
        if dist == 0.0 and vel == 0.0:
            rospy.loginfo('Must be non-zero')
            return False
        
        target_time = dist / vel # [s]
        
        # Twist data
        cmd_vel_ = Twist()
        cmd_vel_.linear.x = vel
        cmd_vel_.angular.z = 0
        
        start_time = time.time()
        end_time = time.time()
        
        rate = rospy.Rate(30)
        rospy.loginfo('start time: ' + str(start_time))
        while end_time - start_time <= target_time:
            self.pub_.publish(cmd_vel_)
            end_time = time.time()
            rate.sleep()
        rospy.loginfo('end time: ' + str(end_time))
        
    def pub_rot_z(self):
        rospy.loginfo('********** pub_rot_z **********')
        # target angle
        theta = rospy.get_param('cmdvel_publisher/theta', 0) # [deg]
        angular_vel = rospy.get_param('cmdvel_publisher/angular_vel', 0) # [deg/s]
        
        rospy.loginfo('theta: ' + str(theta) + ' angular_vel: ' + str(angular_vel))
        
        if theta == 0.0 and angular_vel == 0.0:
            rospy.loginfo('Must be non-zero')
            return False
        
        target_time = theta / angular_vel # [s]
        
        cmd_vel_ = Twist()
        cmd_vel_.linear.x = 0
        cmd_vel_.angular.z = angular_vel * pi / 180.0 # [rad]
        
        start_time = time.time()
        end_time = time.time()
        
        rate = rospy.Rate(30)
        rospy.loginfo('start time: ' + str(start_time))
        while end_time - start_time <= target_time:
            self.pub_.publish(cmd_vel_)
            end_time = time.time()
            rate.sleep()
        rospy.loginfo('end time: ' + str(end_time))

if __name__ == '__main__':
    rospy.init_node('cmdvel_publisher')
    slip_test = slip_test()
    # Go straight
    slip_test.pub_trans_x()
    # Turning
    slip_test.pub_rot_z()