/*
 * tfPointCloud2.cpp
 *
 *  Created on: 2022/07/14
 *      Author: T.Anazawa
 */
#include <ros/ros.h>
#include <sensor_msgs/PointCloud.h>
#include <sensor_msgs/PointCloud2.h>
#include <sensor_msgs/point_cloud_conversion.h>
#include <tf/transform_listener.h>

class PointCloudTfTransform{
	private:
		/*node handle*/
		ros::NodeHandle nh_;
		ros::NodeHandle nhPrivate_;
		/*subscriber*/
		ros::Subscriber pc_sub_;
		/*publisher*/
		ros::Publisher pc2_pub_;
		/*tf*/
		tf::TransformListener tfListener_;
		/*parameters*/
		std::string target_frame;

	public:
		PointCloudTfTransform();
		void callbackPC(const sensor_msgs::PointCloudConstPtr& msg);
		void transformPC(const sensor_msgs::PointCloud& pc2_in);
};

PointCloudTfTransform::PointCloudTfTransform()
	: nhPrivate_("~")
{
	std::cout << "--- pc_tf_transform ---" << std::endl;
	// Parameter
	nhPrivate_.param("target_frame", target_frame, std::string("/map"));
	std::cout << "_target_frame = " << target_frame << std::endl;
	// Subscriber
	pc_sub_ = nh_.subscribe("/waffle_pi/velodyne/point_cloud", 1, &PointCloudTfTransform::callbackPC, this);
	// Publisher
	pc2_pub_ = nh_.advertise<sensor_msgs::PointCloud2>("/points2", 1);
}

void PointCloudTfTransform::callbackPC(const sensor_msgs::PointCloudConstPtr &msg)
{
	transformPC(*msg);
}

void PointCloudTfTransform::transformPC(const sensor_msgs::PointCloud& pc_in)
{
	sensor_msgs::PointCloud pc1_trans;
	sensor_msgs::PointCloud2 pc2_trans;

	try{
		tfListener_.waitForTransform(target_frame, pc_in.header.frame_id, pc_in.header.stamp, ros::Duration(1.0));
		tfListener_.transformPointCloud(target_frame, pc_in.header.stamp, pc_in, pc_in.header.frame_id, pc1_trans);
		sensor_msgs::convertPointCloudToPointCloud2(pc1_trans, pc2_trans);
		pc2_pub_.publish(pc2_trans);
	}
	catch(tf::TransformException ex){
		ROS_ERROR("%s", ex.what());
	}
}

int main(int argc, char** argv)
{
	ros::init(argc, argv, "pc_tf_transform");

	PointCloudTfTransform pc_tf_transform;

	ros::spin();
}



