/*
 * saveMap.cpp
 *
 *  Created date: 2023/09/13
 *      Author: T.Anazawa
 *
 *     Outline: 地図自動保存ノード
 *
 */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ros/ros.h>
#include <ros/package.h>
#include <std_msgs/Bool.h>
#include <uoa_poc3_msgs/r_state.h>

std_msgs::Bool map_save_flg;
const std::string exe_cmd = "rosrun";
const std::string exe_pkg_nm = "map_server";
const std::string output_pkg_nm = "cnoid_turtlebot_navigation";
const std::string pgm_nm = "map_saver";
const std::string output_map = ros::package::getPath(output_pkg_nm) + "/maps/mymap";
std::string entity_id, topic_nm;
const std::string init_val = "";

ros::Subscriber save_map_flg_sub_;
ros::Subscriber robot_state_sub_;
uoa_poc3_msgs::r_state robot_state;

void saveMapFlgCallback(const std_msgs::Bool& msg)
{
	map_save_flg = msg;
	ROS_INFO("map_save_flag: %d", msg.data);
}

void robotStateCallback(const uoa_poc3_msgs::r_state& msg)
{
	/*****************************
	 * string id
	 * string type
	 * string time
	 * string mode
	 * string[] errors
	 * r_pose pose
	 * r_pose_optional destination
	 * float64[36] covariance
	 * r_battery battery
	 *****************************/
	robot_state = msg;
	ROS_INFO("Subscribe robot state msg");
	ROS_INFO("id(%s) type(%s) time(%s) mode(%s) errors.size()(%d)",msg.id.c_str(), msg.type.c_str(), msg.time.c_str(), msg.mode.c_str(), (int)msg.errors.size());
}

void initialize(ros::NodeHandle& nh_)
{
	nh_.param("id", entity_id, init_val);
	if(entity_id.empty()){
		topic_nm = "/map:=" + entity_id + "/map";
	} else {
		topic_nm = "/map:=/" + entity_id + "/map";
	}
}

int main(int argc, char **argv)
{
	ros::init(argc, argv, "saveMap");
	ros::NodeHandle nh_;
	initialize(nh_);
	save_map_flg_sub_ = nh_.subscribe("/map_save_flag", 10, &saveMapFlgCallback);
	robot_state_sub_ = nh_.subscribe("/robot_bridge/"+ entity_id +"/state", 100, &robotStateCallback);
	ros::Rate rate(20);

	while(ros::ok()){
		if(map_save_flg.data){
			ROS_INFO("%s %s %s %s %s %s", exe_cmd.c_str(), exe_pkg_nm.c_str(), pgm_nm.c_str(), "-f", output_map.c_str(), topic_nm.c_str());
			if(execlp(exe_cmd.c_str(), exe_cmd.c_str(), exe_pkg_nm.c_str(), pgm_nm.c_str(), "-f", output_map.c_str(), topic_nm.c_str(), NULL) < 0){
				ROS_ERROR("Failed to start saveMap node.");
				return 1;
			}
		} else if(robot_state.errors.size() != 0){
			for(int i = 0; i < robot_state.errors.size(); i++){
				ROS_ERROR("%s", robot_state.errors[i].c_str());
			}
			return 1;
		}
		ros::spinOnce();
		rate.sleep();
	}

	return 0;
}


